//**********************************************************************************
//  FG4618 Board Demo - Interrupt Driven Serial Communication
//
//  Filename: sr.c
//
//  Description:  This program performs serial communication between hyperterminal
//				  and MSP430FG4618 via RS232 in USCI_A0 mode of the microcontroller.
//	Instructions: 1) Set the following parameters in hyperterminal 
//						Port :			COM1
//						Baud rate :		38400
//						Data bits:		8
//						Parity:		None
//						Stop bits:		1
//						Flow Control:	None	
//				  2) Go to "Project>>Options>>C/C++ compiler>>Preprocessor"	and
//					 add ".\drv\" (without quotes) in "Additional include 
//					 directories:" field. "drv" is the folder which contains
//					 the "include" files for use by this program.
//				  3) Go to "Project>>Options>>Linker>>Config" and add ".\drv\" 
//					 (without quotes) in the "search paths:" field.
//
//  Authors: E. Jovanov and M. Avula
//  		UAH
//  September 2009
//	Built with IAR Embedded Workbench IDE Version: 5.3.6.927
//**********************************************************************************

#include <msp430xG46x.h>
#include <string.h>
#include "serial.h"

#define LED_SETUP()   P1DIR |= BIT0 + BIT1		// P1.0 & P1.1 output to drive LEDs

#define LEDON0     P1OUT |= BIT0				// Switch ON LED 0
#define LEDOFF0    P1OUT &= ~BIT0				// Switch OFF LED 0
#define LEDTOGGLE0 P1OUT ^= ~BIT0				// Toggle LED 0

#define LEDON1     P1OUT |= BIT1				// Switch ON LED 1
#define LEDOFF1    P1OUT &= ~BIT1				// Switch OFF LED1

#define USE_UART0								// Define to use UART0 
#define UART0_INT_DRIVEN						// Define to use UART0 in Interrupt Driven Mode

//Define buffer size 
#define UART0_RX_BUFSIZE  64
#define UART0_TX_BUFSIZE  64

//Delay some time
void delay(void)
{
  unsigned short int i;
  for (i = 65000; i > 0; i--);
  for (i = 65000; i > 0; i--);
}

////////////////////////////////////////////////////////////////
//   USCI_A0 variables
////////////////////////////////////////////////////////////////
#ifdef UART0_INT_DRIVEN
#ifdef USE_UART0
char uart0_rx_buf[UART0_RX_BUFSIZE];
char uart0_tx_buf[UART0_TX_BUFSIZE];

unsigned char uart0_rx_head, uart0_rx_tail;
unsigned char uart0_tx_head, uart0_tx_tail;
unsigned char uart0_flags;

#endif // USE_UART0
#endif // UART0_INT_DRIVEN

////////////////////////////////////////////////////////////////
//   UART0 functions
////////////////////////////////////////////////////////////////
#ifdef USE_UART0
#ifdef UART0_INT_DRIVEN

void UART0_ClearTxBuffer(void)
{
  uart0_tx_head = 0;
  uart0_tx_tail = 0;
}

void UART0_ClearRxBuffer(void)
{
  uart0_rx_head = 0;
  uart0_rx_tail = 0;
}

bool UART0_TxBufferEmpty(void)
{
  return (uart0_tx_head == uart0_tx_tail);
}

bool UART0_RxBufferEmpty(void)
{
  return (uart0_rx_head == uart0_rx_tail);
}

bool UART0_TxBufferFull(void)
{
  unsigned char next = uart0_tx_head + 1;
  if (next >= UART0_TX_BUFSIZE)
    next = 0;

  return (next == uart0_tx_tail);
}

bool UART0_RxBufferFull(void)
{
  unsigned char next = uart0_rx_head + 1;
  if (next >= UART0_RX_BUFSIZE)
    next = 0;

  return (next == uart0_rx_tail);
}
#endif //UART0_INT_DRIVEN


bool UART0_RXready(void)
{
#ifdef UART0_INT_DRIVEN
  return (uart0_rx_head != uart0_rx_tail);
#else
  return ((IFG1 & UCA0RXIFG) != 0);
#endif
}

char UART0_getchar(void)
{
#ifdef UART0_INT_DRIVEN
  char c;

  /* wait forever for character to arrive */
  while (uart0_rx_tail == uart0_rx_head);

  c = uart0_rx_buf[uart0_rx_head];
  if (++uart0_rx_head >= UART0_RX_BUFSIZE)
    uart0_rx_head = 0;
  
  return c;
#else
  for (;;)
  {
    if (UCA0IRRCTL & RXERR)
    {
      // if you want to answer receive errors, do it here.
      UCA0IRRCTL &= ~(FE+PE+OE+BRK+RXERR);
    }
    else
    {
      if (IFG1 & UCARXIFG0)
      {  // we've received a character
        return UCA0RXBUF;
      }
    }
  }
#endif //UART0_INT_DRIVEN
}

char UART0_putchar(char c)
{
#ifdef UART0_INT_DRIVEN
  unsigned char next;

  next = uart0_tx_head + 1;
  if (next >= UART0_TX_BUFSIZE)
    next = 0;

  /* wait forever until buffer is not full */
  /* apr 25 01 - jcc changed to dump bytes instead of waiting */
  //while (next == uart0_tx_tail);
  if (next == uart0_tx_tail)
     return 0;
  uart0_tx_buf[uart0_tx_head] = c;  // store character in buffer
  uart0_tx_head = next;

  // if not already transmitting, then start transmit interrupt
  if (!(uart0_flags & UART_TRANSMITTING))
  {
    UCA0TXBUF = uart0_tx_buf[uart0_tx_tail];
    if (++uart0_tx_tail >= UART0_TX_BUFSIZE)
      uart0_tx_tail = 0;
    uart0_flags |= UART_TRANSMITTING;      /* set transmitting flag */
  }
  
  return c;
#else
  // wait for other character to transmit
  while (!(IFG1 & UCA0TXIFG));
  
  UCA0TXBUF = c;
  return c;
#endif //UART0_INT_DRIVEN
}

void UART0_putstring(char *s)
{
  while (*s) UART0_putchar(*s++);
}

#endif // USE_UART0


////////////////////////////////////////////////////////////////
//   UART0 Initialization									  //
////////////////////////////////////////////////////////////////
void UART_Initialize(void)
{

#ifdef USE_UART0

  P2SEL |= BIT4+BIT5;	// Set UC0TXD and UC0RXD to transmit and receive data	
  UCA0CTL1 |= BIT0;		// software reset
  UCA0CTL0 = 0;			// USCI_A0 control register
  UCA0CTL1 |= UCSSEL_2; // clock source SMCLK
  UCA0BR0=64;			// 2MHz 115200
  UCA0BR1=0;			// 2MHz 115200
  UCA0MCTL=0;			// Modulation
  UCA0CTL1 &= ~BIT0;	// software reset
  	
#ifdef UART0_INT_DRIVEN
  uart0_rx_head = 0;
  uart0_rx_tail = 0;
  uart0_tx_head = 0;
  uart0_tx_tail = 0;
  uart0_flags = 0;
  IE2 = UCA0RXIE + UCA0TXIE;				// enable TX and RX interrupts
#endif // UART0_INT_DRIVEN
#endif // USE_UART0

}  //end UART_Initialize()

#ifdef UART0_INT_DRIVEN
#ifdef USE_UART0

/* RS232 receive interrupt service routine */
#pragma vector=USCIAB0RX_VECTOR
__interrupt void USCI0RX_ISR(void)
{
  unsigned char next;

  /* if we received a bad character, just clear flags and discard. */
  if (UCA0STAT & UCRXERR)			//Check Status register 
    UCA0STAT = 0;					//Clear the status register of any other interrupt routines
  {
    next = uart0_rx_tail + 1;
    if (next >= UART0_RX_BUFSIZE)
      next = 0;

    if (next != uart0_rx_head)
    {
      uart0_rx_buf[uart0_rx_tail] = UCA0RXBUF;
      uart0_rx_tail = next;
      uart0_flags &= ~UART_RX_OVERFLOW;     /* clear RX buffer full flag */
    }
    else
    {
      uart0_flags |= UART_RX_OVERFLOW;      /* set RX buffer full flag */
    }
    LPM0_EXIT;	// exit low power mode and continue in MAIN
  }
}


/* RS232 transmit interrupt service routine */
#pragma vector=USCIAB0TX_VECTOR
__interrupt void USCI0TX_ISR(void)
{
  unsigned int tmp_vec;
  
  tmp_vec=IFG2;
  if (tmp_vec&UCA0TXIFG)
  {
  if (uart0_tx_tail != uart0_tx_head)		/* check if buffer is empty */
  {

    UCA0TXBUF = uart0_tx_buf[uart0_tx_tail];
    if (++uart0_tx_tail >= UART0_TX_BUFSIZE)
      uart0_tx_tail = 0;
    uart0_flags |= UART_TRANSMITTING;       /* set transmitting flag */

  }
  else
  {
    uart0_flags &= ~UART_TRANSMITTING;      /* clear transmitting flag */
    IFG2=0;
  }
  }// end INT USCI_A
}// end ISR

#endif //USE_UART0
#endif // UART0_INT_DRIVEN

////////////////////////////////////////////////////////////////
//   MAIN													  //
////////////////////////////////////////////////////////////////
void main(void)
{

  WDTCTL = WDTPW + WDTHOLD;					// Stop watchdog timer

  // setup clocks
  FLL_CTL0 |= XCAP18PF;                     // Set load capacitance for xtal
  SCFI0 |= FN_2;                            // x2 DCO, 4MHz nominal DCO
  SCFQCTL = 74;                             // (74+1) x 32768 = 2.45Mhz

  LED_SETUP();								// Prepare LEDs for Output
  P1OUT=0;

  delay();									//introduce some delay
  LEDON0;									//Switch ON LED 0
  delay();									//introduce some delay
  LEDOFF0;									//Switch OFF LED 0

  UART_Initialize();						//Initialize UART

  _EINT();								    // enable global interrupts

  // send a message
    UART0_putchar('C');
    UART0_putchar('P');
    UART0_putchar('E');
    UART0_putchar('4');
    UART0_putchar('9');
    UART0_putchar('5');
    UART0_putchar(0xD);
    UART0_putchar(0xA);

  while(1) 
  {
	LPM0;							//enter low power mode
	UART0_putchar(UART0_getchar());			//Waits of the user to input from keyboard
	delay();							//introduce some delay
    } // end infinite loop

} // end main
